unit MainForm;

interface

uses Windows, Messages, NForm, NCmd;

type
  TNkCalMainForm = class(TForm)
    Cmd: TCmdList;
    Year: Integer;
    Month: Integer;
    OldDay: Integer;
    Today: TSystemTime;
    FontName: string;
    FontBold: Boolean;
    FontItalic: Boolean;
    HolidayColor: COLORREF;
    DayColor: array[0..6] of COLORREF;
    TodayColor: COLORREF;
    BGColor: COLORREF;
    FirstWDay: Integer;
    constructor Create(const Caption: string; ALeft, ATop, AWidth,
      AHeight: Integer); override;
    procedure LoadConfig;
    procedure LoadKeyMap;
    procedure DrawCalendar(DC: HDC; Year, Month: Integer);
    procedure UpdateDay;
    procedure DoCmd(const Name: string);
    procedure WMContextMenu(var Message: TMessage); message WM_CONTEXTMENU;
    procedure WMCommand(var Message: TWMCommand); message WM_COMMAND;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMTimer(var Message: TWMTimer); message WM_TIMER;
    procedure WMClose(var Message: TWMClose); message WM_CLOSE;
    procedure WMQueryEndSession(var Message: TWMQueryEndSession);
      message WM_QUERYENDSESSION;
  end;

var
  AppName, ProfDir: string;

implementation

uses NSys, NLib, NDate, NConf, Command, version, holiday_jp;

const
  DayName: array[0..6] of string = (
    'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'
  );

constructor TNkCalMainForm.Create(const Caption: string; ALeft, ATop, AWidth,
  AHeight: Integer);
begin
  inherited;
  SetBackground(GetStockObject(NULL_BRUSH));
  Cmd := TCmdList.Create(CmdList);
  LoadKeyMap;
  SetAccelerator(Cmd.AccelHandle);

  FontName := 'Arial';
  FirstWDay := 6;
  BGColor := RGB(255, 255, 255);
  TodayColor := RGB(230, 234, 242);
  HolidayColor := RGB(255, 0, 0);
  DayColor[0] := RGB(0, 0, 0);
  DayColor[1] := RGB(0, 0, 0);
  DayColor[2] := RGB(0, 0, 0);
  DayColor[3] := RGB(0, 0, 0);
  DayColor[4] := RGB(0, 0, 0);
  DayColor[5] := RGB(0, 0, 255);
  DayColor[6] := RGB(255, 0, 0);

  UpdateDay;
  DoCmd(cmThisMonth);
  SetTimer(Handle, 1, 1000, nil);
end;

procedure TNkCalMainForm.LoadConfig;
var
  s: string;
begin
  try
    with TConf.Create(JoinPath(ProfDir, 'config')) do
      try
        while ReadLine do
        begin
          s := GetToken;
          if s = 'font' then
          begin
            FontName := GetToken;
            s := GetToken;
            FontBold := FontBold or (s = 'bold');
            FontItalic := FontItalic or (s = 'italic');
            s := GetToken;
            FontBold := FontBold or (s = 'bold');
            FontItalic := FontItalic or (s = 'italic');
          end
          else if s = 'firstweekday' then
            FirstWDay := SInt(GetToken);
        end;
      finally
        Free;
      end;
  except
    on E: EIOError do ;
  end;
end;


procedure TNkCalMainForm.UpdateDay;
begin
  OldDay := Today.wDay;
  GetLocalTime(Today);
  if OldDay <> Today.wDay then 
  begin
    SetText(Fmt('%s/%s - %s', [Today.wYear, Today.wMonth, AppName]));
    Invalidate;
  end;
end;

procedure TNkCalMainForm.LoadKeyMap;
var
  i: Integer;
  s: array[0..255] of string;
begin
  for i :=0 to High(s) do
    s[i] := '';
  i := 0;
  try
    with TConf.Create(JoinPath(ProfDir, 'keymap')) do
      try
        while ReadLine and (i < High(s)) do
        begin
          s[i] := GetToken;
          s[i+1] := GetToken;
          Inc(i, 2);
        end;
      finally
        Free;
      end;
  except
    on E: EIOError do ;
  end;
  Cmd.BindKeys(DefaultKeys, s);
end;

procedure TNkCalMainForm.DoCmd(const Name: string);
begin
  if Name = cmStayOnTop then
    SetStayOnTop(not StayOnTop)
  else if Name = cmPrevMonth then
  begin
    if Month = 1 then
    begin
      Month := 12;
      Dec(Year);
    end else
      Dec(Month);
    SetText(Fmt('%s/%s - %s', [Year, Month, AppName]));
    Invalidate;
  end
  else if Name = cmNextMonth then
  begin
    if Month = 12 then
    begin
       Month := 1;
       Inc(Year);
    end else
       Inc(Month);
    SetText(Fmt('%s/%s - %s', [Year, Month, AppName]));
    Invalidate;
  end
  else if Name = cmThisMonth then
  begin
    Year := Today.wYear;
    Month := Today.wMonth; 
    SetText(Fmt('%s/%s - %s', [Year, Month, AppName]));
    Invalidate;
  end
  else if Name = cmZoom100 then
    SetSize(320, 240)
  else if Name = cmZoom200 then
    SetSize(640, 480)
  else if Name = cmZoom300 then
    SetSize(960, 720)
  else if Name = cmAbout then
  begin
    AboutBox(Handle, AppName,
      AppName+' '+VersionStr+' ('+ReleaseDate+')'+EOL+EOL+
      Copyright+EOL+EOL+
      'This software is provided under the zlib/libpng license.'+EOL+EOL+
      'Profile Folder: '+ProfDir);
  end
  else if Name = cmExit then
    Close;
end;

procedure TNkCalMainForm.DrawCalendar(DC: HDC; Year, Month: Integer);
var
  i, j, wd, x, y, w, h, cw, ch: Integer;
  c: COLORREF;
  s: string;
  rc: TRect;
  font, oldfont: HFONT;
  br: HBRUSH;
  fontsize, fw: Integer;
  nnho: Boolean;
begin
  GetClientSize(cw, ch);
  w := cw div 7;
  h := ch div 8;
  x := 0;
  y := 0;
  
  //br := CreateSolidBrush(GetSysColor(COLOR_WINDOW));
  br := CreateSolidBrush(BGColor);
  FillRect(dc, Rect(0, 0, cw, ch), br);
  DeleteObject(br);
  
  fontsize := h * 90 div 100;
  if FontBold then fw := FW_BOLD else fw := FW_NORMAL;
  font := CreateFont(fontsize, 0, 0, 0, fw, Ord(FontItalic), 0, 0,
    DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
    DEFAULT_QUALITY, DEFAULT_PITCH, PChar(FontName));
  oldfont := SelectObject(DC, font);

  SetBkMode(DC, TRANSPARENT);

  // Year and Month
  rc := Rect(0, 0, cw, h);
  if (Year = Today.wYear) and (Month = Today.wMonth) then
  begin
    br := CreateSolidBrush(TodayColor);
    FillRect(DC, rc, br);
    DeleteObject(br);
  end;
  s := Fmt('%s/%s', [Year, Month]);
  DrawText(DC, PChar(s), Length(s), rc,
    DT_CENTER or DT_VCENTER or DT_SINGLELINE);
  Inc(y, h);
 
  // Day Name
  for i := 0 to 6 do
  begin
    j := (FirstWDay+i) mod 7;
    s := DayName[j];
    SetTextColor(DC, DayColor[j]);
    rc := Rect(i*w, y, (i+1)*w, y+h);
    DrawText(DC, PChar(s), Length(s), rc,
      DT_CENTER or DT_VCENTER or DT_SINGLELINE);
  end;

  // Days
  wd := Weekday(Year, Month, 1);
  for i := FirstWDay to FirstWDay + 7 do
    if i mod 7 = wd then
      break
    else
      Inc(x, w);

  Inc(y, h);
  nnho := False; // XXX
  for i := 1 to DaysInMonth(Year, Month) do
  begin
    s := IStr(i);
    rc := Rect(x, y, x+w, y+h);

    if (Year=Today.wYear) and (Month=Today.wMonth) and (i=Today.wDay) then
    begin
      br := CreateSolidBrush(TodayColor);
      FillRect(DC, rc, br);
      DeleteObject(br);
    end;

    c := DayColor[wd];

    if nnho then // Compensatory holiday
    begin
      c := HolidayColor;
      nnho := False;
    end;

    if IsHoliday(Year, Month, i) then // Holiday
    begin
      c := HolidayColor;
      nnho := wd = 6;
    end;

    SetTextColor(DC, c);
    DrawText(DC, PChar(s), Length(s), rc,
      DT_CENTER or DT_VCENTER or DT_SINGLELINE);

    if (FirstWDay+6) mod 7 = wd then
    begin
      x := 0;
      Inc(y, h);
    end else
      Inc(x, w);

    wd := (wd+1) mod 7;
  end;

  SelectObject(DC, oldfont);
  DeleteObject(font);
end; 

procedure TNkCalMainForm.WMContextMenu(var Message: TMessage);
var
  x, y: Integer;
begin
  ScreenToClient(Message.LParamLo, Message.LParamHi, x, y);
  if y < 0 then
  begin
    inherited;
    Exit;
  end;
  with TPopupMenu.Create do
    try
      CreateCmdMenu(Handle, Cmd, MenuItems);
      CheckItem(Cmd.GetId(cmStayOnTop), StayOnTop);
      if Message.LParam = -1 then
      begin
        ClientToScreen(0, 0, x, y);
        Popup(Self.Handle, x, y)
      end else
        Popup(Self.Handle, Message.LParamLo, Message.LParamHi);
    finally
      Free;
    end;
end;

procedure TNkCalMainForm.WMTimer(var Message: TWMTimer);
begin
  UpdateDay;
end;

procedure TNkCalMainForm.WMPaint(var Message: TWMPaint);
var
  ps: TPaintStruct;
  dc: HDC;
begin
  dc := BeginPaint(Handle, ps);
  DrawCalendar(dc, Year, Month);
  EndPaint(Handle, ps);
end;

procedure TNkCalMainForm.WMCommand(var Message: TWMCommand);
begin
  with Message do
    DoCmd(Cmd.GetName(ItemID));
end;

procedure TNkCalMainForm.WMClose(var Message: TWMClose);
begin
  KillTimer(Handle, 1);
  try
    SaveFormPos(ProfDir, Self);
  except
    on E: EIOError do ErrMsgBox(AppName, E.Path+EOL+EOL+E.Msg);
  end;
  inherited;
end;

procedure TNkCalMainForm.WMQueryEndSession(var Message: TWMQueryEndSession);
begin
  try
    SaveFormPos(ProfDir, Self);
  except
    on E: EIOError do ErrMsgBox(AppName, E.Path+EOL+EOL+E.Msg);
  end;
  Message.Result := Integer(True);
end;

end.
